/**
 * Ext GWT 3.0.0-beta2 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.google.gwt.dom.client.Element;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.event.shared.SimpleEventBus;
import com.sencha.gxt.core.shared.FastMap;
import com.sencha.gxt.widget.core.client.event.RegisterEvent;
import com.sencha.gxt.widget.core.client.event.RegisterEvent.HasRegisterHandlers;
import com.sencha.gxt.widget.core.client.event.RegisterEvent.RegisterHandler;
import com.sencha.gxt.widget.core.client.event.UnregisterEvent;
import com.sencha.gxt.widget.core.client.event.UnregisterEvent.HasUnregisterHandlers;
import com.sencha.gxt.widget.core.client.event.UnregisterEvent.UnregisterHandler;

/**
 * Provides a registry of all attached {@link Component}. Components
 * automatically register and unregister with the widget manager when they
 * are attached and detached.
 * 
 * <p />
 * Attaching and detaching are parts of the GWT widget lifecycle. Basically,
 * widgets are attached when they become part of the DOM and then detached when
 * removed from the DOM.
 * 
 * <dl>
 * <dt><b>Events:</b></dt>
 * 
 * <dd>{@link RegisterEvent}</b><br>
 * <dd>{@link UnregisterEvent}</b><br>
 * </dl>
 */
public class ComponentManager implements HasRegisterHandlers<Component>, HasUnregisterHandlers<Component> {

  private static ComponentManager instance = new ComponentManager();

  /**
   * Returns the widget manager instance.
   * 
   * @return the widget manager
   */
  public static ComponentManager get() {
    return instance;
  }

  private SimpleEventBus eventBus;
  private Map<String, Component> map;

  private ComponentManager() {
    map = new FastMap<Component>();
  }

  @Override
  public HandlerRegistration addRegisterHandler(RegisterHandler<Component> handler) {
    return ensureHandlers().addHandler(RegisterEvent.getType(), handler);
  }

  @Override
  public HandlerRegistration addUnregisterHandler(UnregisterHandler<Component> handler) {
    return ensureHandlers().addHandler(UnregisterEvent.getType(), handler);
  }

  /**
   * Attempts to find a widget.
   * 
   * @param target the element or inner element of the widget
   * @return the matching widget or null if no match
   */
  @SuppressWarnings("unchecked")
  public <X extends Component> X find(Element target) {
    return (X) find(target, null);
  }

  /**
   * Attempts to find a widget.
   * 
   * @param target the element or inner element of the widget
   * @param clazz the class the widget should have
   * @return the matching widget or null if no match
   */
  @SuppressWarnings("unchecked")
  public <X extends Component> X find(Element target, Class<X> clazz) {
    while (target != null) {
      Component c = map.get(target.getId());
      if (c != null && (clazz == null || c.getClass().equals(clazz))) {
        return (X) c;
      } else {
        target = (Element) target.getParentElement();
      }

    }
    return null;
  }

  /**
   * Returns all widget by class.
   * 
   * @param clazz the class to match
   * @return the list of matching components
   */
  @SuppressWarnings("unchecked")
  public <X extends Component> List<X> get(Class<?> clazz) {
    List<X> temp = new ArrayList<X>();
    for (Component c : map.values()) {
      if (c.getClass() == clazz) {
        temp.add((X) c);
      }
    }
    return temp;
  }

  /**
   * Returns a widget by id.
   * 
   * @param id the widget id
   * @return the widget or null if no match
   */
  public Component get(String id) {
    return map.get(id);
  }

  /**
   * Returns a collection of all registered components.
   * 
   * @return the collection of components
   */
  public Collection<Component> getAll() {
    return map.values();
  }

  SimpleEventBus ensureHandlers() {
    return eventBus == null ? eventBus = new SimpleEventBus() : eventBus;
  }

  void register(Component component) {
    assert component != null : "widget cannot be null";
    map.put(component.getId(), component);
    ensureHandlers().fireEventFromSource(new RegisterEvent<Component>(component), this);
  }

  void unregister(Component component) {
    assert component != null : "widget cannot be null";
    map.remove(component.getId());
    ensureHandlers().fireEventFromSource(new UnregisterEvent<Component>(component), this);
  }

}