/**
 * Ext GWT 3.0.0-beta2 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client;

import java.util.ArrayList;
import java.util.List;

import com.sencha.gxt.messages.client.DefaultMessages;
import com.sencha.gxt.widget.core.client.button.TextButton;
import com.sencha.gxt.widget.core.client.event.SelectEvent;
import com.sencha.gxt.widget.core.client.event.SelectEvent.SelectHandler;

/**
 * A <code>Window</code> with specialized support for buttons. Defaults to a
 * dialog with an 'ok' button.</p>
 * 
 * Code snippet:
 * 
 * <pre>
 * Dialog d = new Dialog();
 * d.setHeading(&quot;Exit Warning!&quot;);
 * d.addText(&quot;Do you wish to save before exiting?&quot;);
 * d.setBodyStyle(&quot;fontWeight:bold;padding:13px;&quot;);
 * d.setSize(300, 100);
 * d.setHideOnButtonClick(true);
 * d.setButtons(Dialog.YESNOCANCEL);
 * d.show();
 * </pre>
 * 
 * <p />
 * The predefined buttons can be retrieved from the button bar using their
 * respective ids ('ok', 'cancel', 'yes', 'no', 'cancel') or by index. The
 * method {@link #getButtonBar()} creates the buttons, so any call before the
 * dialog is rendered will cause the buttons to be created based on the
 * {@link #setPredefinedButtons(PredefinedButton...)} value.
 */
public class Dialog extends Window {

  public interface DialogMessages {

    String cancel();

    String close();

    String no();

    String ok();

    String yes();

  }

  public enum PredefinedButton {
    OK, CANCEL, CLOSE, YES, NO;
  }

  protected class DefaultDialogMessages implements DialogMessages {

    public String cancel() {
      return DefaultMessages.getMessages().messageBox_cancel();
    }

    public String close() {
      return DefaultMessages.getMessages().messageBox_close();
    }

    public String no() {
      return DefaultMessages.getMessages().messageBox_no();
    }

    public String ok() {
      return DefaultMessages.getMessages().messageBox_ok();
    }

    public String yes() {
      return DefaultMessages.getMessages().messageBox_yes();
    }

  }

  private boolean hideOnButtonClick = false;

  private List<PredefinedButton> buttons = new ArrayList<PredefinedButton>();
  private SelectHandler handler = new SelectHandler() {

    @Override
    public void onSelect(SelectEvent event) {
      onButtonPressed((TextButton) event.getSource());
    }
  };

  private DialogMessages dialogMessages;

  public Dialog() {
    setPredefinedButtons(PredefinedButton.OK);
  }

  public Dialog(WindowAppearance appearance) {
    super(appearance);
    setPredefinedButtons(PredefinedButton.OK);
  }

  public TextButton getButtonById(String string) {
    return (TextButton) buttonBar.getItemByItemId(string);
  }

  public DialogMessages getDialogMessages() {
    if (dialogMessages == null) {
      dialogMessages = new DefaultDialogMessages();
    }
    return dialogMessages;
  }

  /**
   * Returns the button's.
   * 
   * @return the buttons the buttons
   */
  public List<PredefinedButton> getPredefinedButtons() {
    return buttons;
  }

  /**
   * Returns true if the dialog will be hidden on any button click.
   * 
   * @return the hide on button click state
   */
  public boolean isHideOnButtonClick() {
    return hideOnButtonClick;
  }

  public void setDialogMessages(DialogMessages dialogMessages) {
    this.dialogMessages = dialogMessages;
  }

  /**
   * True to hide the dialog on any button click.
   * 
   * @param hideOnButtonClick true to hide
   */
  public void setHideOnButtonClick(boolean hideOnButtonClick) {
    this.hideOnButtonClick = hideOnButtonClick;
  }

  /**
   * Sets the predefined buttons to display (defaults to OK). Can be any
   * combination of:
   * 
   * <pre>
   * {@link PredefinedButton#OK}
   * {@link PredefinedButton#CANCEL}
   * {@link PredefinedButton#CLOSE}
   * {@link PredefinedButton#YES}
   * {@link PredefinedButton#NO}
   * </pre>
   */
  public void setPredefinedButtons(PredefinedButton... buttons) {
    this.buttons.clear();
    for (PredefinedButton b : buttons) {
      this.buttons.add(b);
    }

    createButtons();
  }

  /**
   * Creates the buttons based on button creation constant
   */
  protected void createButtons() {
    getButtonBar().clear();
    setFocusWidget(null);

    for (int i = 0; i < buttons.size(); i++) {
      PredefinedButton b = buttons.get(i);
      TextButton tb = new TextButton(getText(b));
      tb.setItemId(b.name());
      tb.addSelectHandler(handler);
      if (i == 0) {
        setFocusWidget(tb);
      }
      addButton(tb);
    }
  }

  protected String getText(PredefinedButton button) {
    switch (button) {
      case OK:
        return getDialogMessages().ok();
      case CANCEL:
        return getDialogMessages().cancel();
      case CLOSE:
        return getDialogMessages().close();
      case YES:
        return getDialogMessages().yes();
      case NO:
        return getDialogMessages().no();
      default:
        // Should never happen
        throw new IllegalArgumentException("No text available for this button");
    }
  }

  /**
   * Called after a button in the button bar is selected. If
   * {@link #setHideOnButtonClick(boolean)} is true, hides the dialog when any
   * button is pressed.
   * 
   * @param button the button
   */
  protected void onButtonPressed(TextButton button) {
    if (button == getButtonBar().getItemByItemId(PredefinedButton.CLOSE.name()) || hideOnButtonClick) {
      hide(button);
    }
  }

}
