/**
 * Ext GWT 3.0.0-beta2 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client.form;

import com.google.gwt.core.client.GWT;
import com.google.gwt.safehtml.client.HasSafeHtml;
import com.google.gwt.safehtml.shared.SafeHtml;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.google.gwt.user.client.ui.HasHTML;
import com.google.gwt.user.client.ui.HasText;
import com.google.gwt.user.client.ui.Widget;
import com.sencha.gxt.core.client.GXT;
import com.sencha.gxt.core.client.dom.XDOM;
import com.sencha.gxt.core.client.dom.XElement;
import com.sencha.gxt.core.client.util.Size;
import com.sencha.gxt.widget.core.client.Component;
import com.sencha.gxt.widget.core.client.container.SimpleContainer;
import com.sencha.gxt.widget.core.client.form.FormPanel.LabelAlign;

/**
 * Form field wrapper to add a label and validation error text.
 */
public class FieldLabel extends SimpleContainer implements HasText, HasHTML, HasSafeHtml {

  /**
   * Describes the appearance for a {@link FieldLabel} object. Different from
   * most appearance objects, as it must provide direct access to the wrapper
   * element instead of just access to set and get the properties of the UI.
   */
  public interface FieldLabelAppearance {
    XElement getChildElementWrapper(XElement parent);

    XElement getLabelElement(XElement parent);

    void onLabelAlign(XElement parent, FieldLabel label);

    void render(SafeHtmlBuilder sb, String id, FieldLabel label);
  }

  private final FieldLabelAppearance appearance;

  private String labelSeparator = ":";
  private int labelWidth = 100;
  private int labelPad = 5;
  private LabelAlign labelAlign = LabelAlign.LEFT;

  public FieldLabel() {
    this(null);
  }

  public FieldLabel(Widget widget) {
    this(widget, GWT.<FieldLabelAppearance> create(FieldLabelAppearance.class));
  }

  public FieldLabel(Widget widget, FieldLabelAppearance appearance) {
    super(true);
    this.appearance = appearance;

    String id = widget == null ? "" : widget instanceof Component ? ((Component)widget).getId() : widget.getElement().getId();
    
    if (widget instanceof ValueBaseField<?>) {
      id += "-input";
    }

    SafeHtmlBuilder sb = new SafeHtmlBuilder();
    appearance.render(sb, id, this);
    setElement(XDOM.create(sb.toSafeHtml()));

    if (widget != null) {
      setWidget(widget);
    }
  }
  
  public FieldLabel(Widget widget, String label) {
    this(widget);
    setText(label);
  }

  public FieldLabel(Widget widget, String label, FieldLabelAppearance appearance) {
    this(widget, appearance);
    setText(label);
  }

  @Override
  public String getHTML() {
    return appearance.getLabelElement(getElement()).getInnerHTML();
  }

  public LabelAlign getLabelAlign() {
    return labelAlign;
  }

  public int getLabelPad() {
    return labelPad;
  }

  /**
   * Returns the label separator.
   * 
   * @return the label separator
   */
  public String getLabelSeparator() {
    return labelSeparator;
  }

  /**
   * Returns the label width.
   * 
   * @return the label width
   */
  public int getLabelWidth() {
    return labelWidth;
  }

  @Override
  public String getText() {
    return appearance.getLabelElement(getElement()).getInnerText();
  }

  @Override
  public void setHTML(SafeHtml html) {
    setHTML(html.asString());
  }

  @Override
  public void setHTML(String html) {
    appearance.getLabelElement(getElement()).setInnerHTML(html);
  }

  public void setLabelAlign(LabelAlign labelAlign) {
    this.labelAlign = labelAlign;
    appearance.onLabelAlign(getElement(), this);
  }

  public void setLabelPad(int labelPad) {
    this.labelPad = labelPad;
  }

  /**
   * The standard separator to display after the text of each form label
   * (defaults to colon ':').
   * 
   * @param labelSeparator the label separator or "" for none
   */
  public void setLabelSeparator(String labelSeparator) {
    this.labelSeparator = labelSeparator;
  }

  /**
   * Sets the label width (defaults to 100).
   * 
   * @param labelWidth the label width
   */
  public void setLabelWidth(int labelWidth) {
    this.labelWidth = labelWidth;
  }

  @Override
  public void setText(String text) {
    appearance.getLabelElement(getElement()).setInnerText(text + labelSeparator);
  }

  @Override
  public void setWidget(Widget w) {
    super.setWidget(w);
    String id = widget == null ? "" : widget instanceof Component ? ((Component)widget).getId() : widget.getElement().getId();
    
    if (widget instanceof ValueBaseField<?>) {
      id += "-input";
    }
    
    XElement label = getElement().selectNode("label");
    if (label != null) {
      label.setPropertyString("for", id);
    }
  }

  @Override
  protected void doLayout() {
    if (widget != null) {
      Size size = getElement().getStyleSize();
      int width = -1;
      if (!isAutoWidth()) {

        if (labelAlign == LabelAlign.TOP) {
          width = size.getWidth() - getLeftRightMargins(widget);
        } else {
          XElement wrapper = appearance.getChildElementWrapper(getElement());
          width = (wrapper != null ? wrapper.getWidth(true) : size.getWidth()) - getLeftRightMargins(widget);
        }

        if (GXT.isIE6()) {
          width -= 5;
        }
      }
      int height = -1;

      if (!isAutoHeight()) {
        height = size.getHeight() - getTopBottomMargins(widget);
      }

      applyLayout(widget, width, height);
    }
  }

  @Override
  protected XElement getContainerTarget() {
    return appearance.getChildElementWrapper(getElement());
  }

  @Override
  protected void onAfterFirstAttach() {
    super.onAfterFirstAttach();
    String id = widget instanceof Component ? ((Component) widget).getId() : widget.getElement().getId();
    appearance.getLabelElement(getElement()).setAttribute("for", id + "-input");
  }

}
