/**
 * Ext GWT 3.0.0-beta2 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client.form;

import java.text.ParseException;

import com.google.gwt.i18n.client.LocaleInfo;
import com.google.gwt.i18n.client.NumberFormat;
import com.google.gwt.i18n.client.constants.NumberConstants;

public abstract class NumberPropertyEditor<N extends Number> extends PropertyEditor<N> {
  public static class LongPropertyEditor extends NumberPropertyEditor<Long> {
    public LongPropertyEditor() {
      super(1l);
    }

    public LongPropertyEditor(NumberFormat format) {
      super(format, 1l);
    }

    @Override
    public Long doIncr(Long value) {
      return value + getIncrement();
    }

    @Override
    public Long doDecr(Long value) {
      return value - getIncrement();
    }

    @Override
    protected Long parseString(String string) {
      return Long.parseLong(string);
    }

    @Override
    protected Long returnTypedValue(Number number) {
      return number.longValue();
    }
  }
  public static class IntegerPropertyEditor extends NumberPropertyEditor<Integer> {
    public IntegerPropertyEditor() {
      super(1);
    }

    public IntegerPropertyEditor(NumberFormat format) {
      super(format, 1);
    }

    @Override
    public Integer doIncr(Integer value) {
      return value + getIncrement();
    }

    @Override
    public Integer doDecr(Integer value) {
      return value - getIncrement();
    }

    @Override
    protected Integer parseString(String string) {
      return Integer.parseInt(string);
    }

    @Override
    protected Integer returnTypedValue(Number number) {
      return number.intValue();
    }
  }
  public static class ShortPropertyEditor extends NumberPropertyEditor<Short> {
    public ShortPropertyEditor() {
      super((short) 1);
    }

    public ShortPropertyEditor(NumberFormat format) {
      super(format, (short) 1);
    }

    @Override
    public Short doIncr(Short value) {
      return (short) (value + getIncrement());
    }

    @Override
    public Short doDecr(Short value) {
      return (short) (value - getIncrement());
    }

    @Override
    protected Short parseString(String string) {
      return Short.parseShort(string);
    }

    @Override
    protected Short returnTypedValue(Number number) {
      return number.shortValue();
    }
  }
  public static class DoublePropertyEditor extends NumberPropertyEditor<Double> {
    public DoublePropertyEditor() {
      super(1d);
    }

    public DoublePropertyEditor(NumberFormat format) {
      super(format, 1d);
    }

    @Override
    public Double doIncr(Double value) {
      return value + getIncrement();
    }

    @Override
    public Double doDecr(Double value) {
      return value - getIncrement();
    }

    @Override
    protected Double parseString(String string) {
      return Double.parseDouble(string);
    }

    @Override
    protected Double returnTypedValue(Number number) {
      return number.doubleValue();
    }
  }
  public static class FloatPropertyEditor extends NumberPropertyEditor<Float> {
    public FloatPropertyEditor() {
      super(1f);
    }

    public FloatPropertyEditor(NumberFormat format) {
      super(format, 1f);
    }

    @Override
    public Float doIncr(Float value) {
      return value + getIncrement();
    }

    @Override
    public Float doDecr(Float value) {
      return value - getIncrement();
    }

    @Override
    protected Float parseString(String string) {
      return Float.parseFloat(string);
    }

    @Override
    protected Float returnTypedValue(Number number) {
      return number.floatValue();
    }
  }

  protected NumberConstants numbers;
  protected NumberFormat format;
  protected String alpahRegex = "[a-zA-Z]";
  protected String currencySymbolRegex = "\\$";
  protected String groupSeparator;

  private N incrAmount;

  private boolean stripCurrencySymbol;
  private boolean stripAlphas;
  private boolean stripGroupSeparator;

  /**
   * Creates a new number property editor with the default number type (Double).
   */
  public NumberPropertyEditor(N incrAmount) {
    this.incrAmount = incrAmount;
    numbers = LocaleInfo.getCurrentLocale().getNumberConstants();
    groupSeparator = numbers.groupingSeparator();
  }

  /**
   * Creates a new number property editor.
   * 
   * @param format the number format
   */
  public NumberPropertyEditor(NumberFormat format, N incrAmount) {
    this(incrAmount);
    this.format = format;
  }

  /**
   * Creates a new number property editor.
   * 
   * @param pattern the number format pattern
   */
  public NumberPropertyEditor(String pattern, N incrAmount) {
    this(NumberFormat.getFormat(pattern), incrAmount);
  }

  /**
   * Returns the editor's format.
   * 
   * @return the number format
   */
  public NumberFormat getFormat() {
    return format;
  }

  protected abstract N parseString(String string);

  @Override
  public N parse(CharSequence text) throws ParseException {
    String value = text.toString();

    // first try to create a typed value directly from the raw text
    try {
      return parseString(value);
    } catch (Exception e) {
    }

    // second, strip all unwanted characters
    String stripValue = stripValue(value);
    try {
      return parseString(stripValue);
    } catch (Exception e) {
    }

    try {
      // third try parsing with the formatter
      if (format != null) {
        Double d = format.parse(value);
        return (N) returnTypedValue(d);
      } else {
        Double d = NumberFormat.getDecimalFormat().parse(value);
        return (N) returnTypedValue(d);
      }
    } catch (Exception ex) {
      throw new ParseException(ex.getMessage(), 0);
    }
  }

  @Override
  public String render(Number value) {
    if (format != null) {
      return format.format(value.doubleValue());
    }
    return value.toString();
  }

  /**
   * Sets the editor's format.
   * 
   * @param format the format
   */
  public void setFormat(NumberFormat format) {
    this.format = format;
  }

  protected abstract N returnTypedValue(Number number);

  protected String stripValue(String value) {
    if (stripCurrencySymbol) {
      value = value.replaceAll(currencySymbolRegex, "");
    }
    if (stripAlphas) {
      value = value.replaceAll(alpahRegex, "");
    }
    if (stripGroupSeparator) {
      value = value.replaceAll(groupSeparator, "");
    }
    return value;
  }

  public N incr(N value) {
    if (value == null) {
      return getIncrement();
    }
    return doIncr(value);
  }

  public N decr(N value) {
    if (value == null) {
      return doDecr(doDecr(getIncrement()));
    }
    return doDecr(value);
  }

  protected abstract N doIncr(N value);

  protected abstract N doDecr(N value);

  public N getIncrement() {
    return incrAmount;
  }

  public void setIncrement(N value) {
    this.incrAmount = value;
  }
}
