/**
 * Ext GWT 3.0.0-beta2 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client.grid;

import com.google.gwt.cell.client.Cell;
import com.google.gwt.safehtml.shared.SafeHtml;
import com.google.gwt.safehtml.shared.SafeHtmlUtils;
import com.google.gwt.user.client.ui.HasHorizontalAlignment.HorizontalAlignmentConstant;
import com.google.gwt.user.client.ui.Widget;
import com.sencha.gxt.core.client.ValueProvider;

/**
 * A column config for a column in a column model.
 * 
 * <p />
 * The column config is a configuration object that should only be used when
 * creating a column model. After the column model is created, any changes
 * should be made to the column model, not the column config.
 */
public class ColumnConfig<M, N> {

  /**
   * True to disable keyboard column navigation of this column.
   */
  protected boolean ariaIgnore;

  private HorizontalAlignmentConstant alignment;
  private Cell<N> cell;
  private String columnStyleName;
  private boolean fixed;
  private boolean groupable = true;
  private SafeHtml header;
  private boolean hidden;
  private boolean menuDisabled;
  private boolean resizable = true;
  private boolean rowHeader;
  private boolean sortable = true;
  private String style;
  private SafeHtml toolTip;
  private final ValueProvider<M, N> valueProvider;
  private Widget widget;

  private int width = 100;
  private String columnClassName;

  /**
   * Creates a new column config.
   */
  public ColumnConfig(ValueProvider<M, N> valueProvider) {
    this.valueProvider = valueProvider;
  }

  /**
   * Creates a new column config.
   * 
   * @param valueProvider the value provider
   * @param width the column width
   */
  public ColumnConfig(ValueProvider<M, N> valueProvider, int width) {
    this(valueProvider);
    this.width = width;
  }

  /**
   * Creates a new column config.
   * 
   * @param valueProvider the value provider
   * @param width the column width
   * @param name the column name
   */
  public ColumnConfig(ValueProvider<M, N> valueProvider, int width, SafeHtml name) {
    this(valueProvider, width);
    this.header = name;
  }

  /**
   * Creates a new column config.
   * 
   * @param valueProvider thevalueProvider
   * @param width the column width
   * @param name the heading
   */
  public ColumnConfig(ValueProvider<M, N> valueProvider, int width, String name) {
    this(valueProvider, width, SafeHtmlUtils.fromString(name));
  }

  /**
   * Returns the column's horizontal alignment.
   * 
   * @return the horizontal alignment
   */
  public HorizontalAlignmentConstant getAlignment() {
    return alignment;
  }

  /**
   * Returns the column's cell renderer.
   * 
   * @return the renderer
   */
  public Cell<N> getCell() {
    return cell;
  }

  /**
   * Returns the columns class name.
   * 
   * @return the column class name
   */
  public String getColumnClassName() {
    return columnClassName;
  }

  /**
   * Returns the column style name.
   * 
   * @return the column style name
   */
  public String getColumnStyleName() {
    return columnStyleName;
  }

  /**
   * Returns the column's header text.
   * 
   * @return the header text
   */
  public SafeHtml getHeader() {
    return header;
  }

  /**
   * Returns the value provider path.
   * 
   * @return the path
   */
  public String getPath() {
    return valueProvider.getPath();
  }

  /**
   * Returns the column style.
   * 
   * @return the style
   */
  public String getStyle() {
    return style;
  }

  /**
   * Returns the column's tool tip.
   * 
   * @return the tool tip
   */
  public SafeHtml getToolTip() {
    return toolTip;
  }

  /**
   * Returns the column's valueProvider.
   * 
   * @return the valueProvider
   */
  public ValueProvider<M, N> getValueProvider() {
    return valueProvider;
  }

  /**
   * Returns the column's widget.
   * 
   * @return the widget
   */
  public Widget getWidget() {
    return widget;
  }

  /**
   * Returns the column's width.
   * 
   * @return the column width
   */
  public int getWidth() {
    return width;
  }

  /**
   * Returns true if the column width cannot be changed. Applies to both column
   * width calculations (auto fill, force fit, auto expand column) and user
   * resizing.
   * 
   * @return the fixed state
   */
  public boolean isFixed() {
    return fixed;
  }

  /**
   * Returns true if the column can be grouped.
   * 
   * @return true if groupable
   */
  public boolean isGroupable() {
    return groupable;
  }

  /**
   * Returns true if the column is hidden.
   * 
   * @return the hidden state
   */
  public boolean isHidden() {
    return hidden;
  }

  /**
   * Returns true if the column's menu is disabled.
   * 
   * @return the menu disabled state
   */
  public boolean isMenuDisabled() {
    return menuDisabled;
  }

  /**
   * Returns true if the column is resizable.
   * 
   * @return the resizable state
   */
  public boolean isResizable() {
    return resizable;
  }

  /**
   * Returns true if the column is marked as the row header.
   * 
   * @return true if row header
   */
  public boolean isRowHeader() {
    return rowHeader;
  }

  /**
   * Returns <code>true</code> if the column is sortable (pre-render).
   * 
   * @return the sortable state
   */
  public boolean isSortable() {
    return sortable;
  }

  /**
   * Sets the column's horizontal alignment.
   * 
   * @param alignment the alignment
   */
  public void setAlignment(HorizontalAlignmentConstant alignment) {
    this.alignment = alignment;
  }

  /**
   * Sets the column's cell renderer (pre-render).
   * 
   * @param cell the cell renderer
   */
  public void setCell(Cell<N> cell) {
    this.cell = cell;
  }

  /**
   * If provided, this value will be appended to 'x-grid3-td-' and be added to
   * all table cells for the column.
   * 
   * @param columnClassName the column class name
   */
  public void setColumnClassName(String columnClassName) {
    this.columnClassName = columnClassName;
  }

  /**
   * Sets the style name to be applied to the header element (defaults to null).
   * The style is only applied to the header cell, not the data rows.
   * 
   * @param columnStyleName the CSS style name
   */
  public void setColumnStyleName(String columnStyleName) {
    this.columnStyleName = columnStyleName;
  }

  /**
   * True if the column width cannot be changed either by column model or user
   * resizing (defaults to false, pre-render).
   * 
   * @param fixed true for fixed column width
   */
  public void setFixed(boolean fixed) {
    this.fixed = fixed;
  }

  /**
   * Sets if the column is groupable (defaults to true). Only applies when using
   * a GroupingView.
   * 
   * @param groupable true to allow grouping
   */
  public void setGroupable(boolean groupable) {
    this.groupable = groupable;
  }

  /**
   * Sets the column's header text.
   * 
   * @see #setHeader(String)
   * @param header the header text
   */
  public void setHeader(SafeHtml header) {
    this.header = header;
  }

  /**
   * Sets the column's header text, to be displayed as escaped html.
   * 
   * @see #setHeader(SafeHtml)
   * @param header the header text to be displayed as escaped html
   */
  public void setHeader(String header) {
    setHeader(SafeHtmlUtils.fromString(header));
  }

  /**
   * Sets whether the column is hidden.
   * 
   * @param hidden true to hide
   */
  public void setHidden(boolean hidden) {
    this.hidden = hidden;
  }

  /**
   * Sets whether the column's menu is disabled (pre-render).
   * 
   * @param menuDisabled true to disable the menu
   */
  public void setMenuDisabled(boolean menuDisabled) {
    this.menuDisabled = menuDisabled;
  }

  /**
   * Specifies if the column may be resized (defaults to true, pre-render).
   * 
   * @param resizable the resizable state
   */
  public void setResizable(boolean resizable) {
    this.resizable = resizable;
  }

  /**
   * True to mark this column as the row header. Only applicable when ARIA is
   * enabled for accessibility. Screen reader will annouce the columns value
   * when using a cell selection model.
   * 
   * @param rowHeader true to mark the column as the row header
   */
  public void setRowHeader(boolean rowHeader) {
    this.rowHeader = rowHeader;
  }

  /**
   * Sets if the column can be sorted (defaults to true, pre-render).
   * 
   * @param sortable the sortable state
   */
  public void setSortable(boolean sortable) {
    this.sortable = sortable;
  }

  /**
   * Sets one to many styles.
   * 
   * <pre>
   * &lt;code&gt;
   *        setStyle(&quot;color:red;font-weight: bold&quot;);
   * &lt;/code&gt;
   * </pre>
   * 
   * @param style the style
   */
  public void setStyle(String style) {
    this.style = style;
  }

  /**
   * Sets the column's tool tip.
   * 
   * @param toolTip the tool tip
   */
  public void setToolTip(SafeHtml toolTip) {
    this.toolTip = toolTip;
  }

  /**
   * Sets the column's widget.
   * 
   * @param widget the widget
   * @param header the text used for the column context menu
   */
  public void setWidget(Widget widget, SafeHtml header) {
    this.widget = widget;
    this.header = header;
  }

  /**
   * Sets the column's width in pixels.
   * 
   * @param width the width
   */
  public void setWidth(int width) {
    this.width = width;
  }
}
